# Copyright (c) 2008, Michigan State University.

"""
Defines some constant strings identifications used in multiple
files throughout screed
"""

# Name of table holding information about rest of db
_SCREEDADMIN = 'SCREEDADMIN'

# Names of _SCREEDADMIN columns
_FIELDNAME = 'FIELDNAME'
_ROLENAME = 'ROLE'
_PRIMARY_KEY = 'id'

# Names of roles
_STANDARD_TEXT = 'STANDARDATTR'
_SLICEABLE_TEXT = 'SLICEABLEATTR'
_INDEXED_TEXT_KEY = 'TEXTKEYATTR'
_PRIMARY_KEY_ROLE = 'INTKEYATTR'

# Name of table holding sequence information
_DICT_TABLE = 'DICTIONARY_TABLE'

# The file extension given to all screed databases
fileExtension = '_screed'
